from __future__ import annotations

import asyncio
from enum import Enum, auto
import logging

from homeassistant.components.alarm_control_panel import (
    AlarmControlPanelEntity,
    AlarmControlPanelEntityFeature,
)
from homeassistant.config_entries import ConfigEntry
from homeassistant.const import STATE_ALARM_ARMED_AWAY, STATE_ALARM_ARMED_HOME, STATE_ALARM_DISARMED, STATE_ALARM_TRIGGERED
from homeassistant.core import HomeAssistant
from homeassistant.helpers import entity_platform
from homeassistant.helpers.entity_platform import AddEntitiesCallback

from .const import COORDINATOR, DOMAIN, Schema
from .coordinator import EufySecurityDataUpdateCoordinator
from .entity import EufySecurityEntity
from .eufy_security_api.const import MessageField
from .eufy_security_api.metadata import Metadata
from .eufy_security_api.util import get_child_value

_LOGGER: logging.Logger = logging.getLogger(__package__)


class CurrentModeToState(Enum):
    """Alarm Entity Mode to State"""

    NONE = -1
    AWAY = 0
    HOME = 1
    CUSTOM_BYPASS = 3
    NIGHT = 4
    VACATION = 5
    DISARMED = 63


class CurrentModeToStateValue(Enum):
    """Alarm Entity Mode to State Value"""

    NONE = "Unknown"
    AWAY = STATE_ALARM_ARMED_AWAY
    HOME = STATE_ALARM_ARMED_HOME
    CUSTOM_BYPASS = auto()
    NIGHT = auto()
    VACATION = auto()
    DISARMED = STATE_ALARM_DISARMED
    TRIGGERED = STATE_ALARM_TRIGGERED
    ALARM_DELAYED = "Alarm delayed"


CUSTOM_CODES = [3, 4, 5]


async def async_setup_entry(hass: HomeAssistant, config_entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None:
    """Setup alarm control panel entities."""
    coordinator: EufySecurityDataUpdateCoordinator = hass.data[DOMAIN][COORDINATOR]
    product_properties = []
    for product in coordinator.api.stations.values():
        if product.has(MessageField.GUARD_MODE.value) is True:
            product_properties.append(product.metadata[MessageField.CURRENT_MODE.value])

    entities = [EufySecurityAlarmControlPanel(coordinator, metadata) for metadata in product_properties]
    async_add_entities(entities)
    # register entity level services
    platform = entity_platform.async_get_current_platform()
    platform.async_register_entity_service(
        "trigger_alarm_with_duration", Schema.TRIGGER_ALARM_SERVICE_SCHEMA.value, "async_alarm_trigger_with_duration"
    )
    platform.async_register_entity_service("reset_alarm", {}, "async_reset_alarm")
    platform.async_register_entity_service("alarm_arm_custom1", {}, "async_alarm_arm_custom_bypass")
    platform.async_register_entity_service("alarm_arm_custom2", {}, "async_alarm_arm_night")
    platform.async_register_entity_service("alarm_arm_custom3", {}, "async_alarm_arm_vacation")


class EufySecurityAlarmControlPanel(AlarmControlPanelEntity, EufySecurityEntity):
    """Base alarm control panel entity for integration"""

    def __init__(self, coordinator: EufySecurityDataUpdateCoordinator, metadata: Metadata) -> None:
        super().__init__(coordinator, metadata)
        self._attr_name = f"{self.product.name}"
        self._attr_icon = None
        self._attr_code_arm_required = False
        self._attr_supported_features = (
            AlarmControlPanelEntityFeature.ARM_HOME | AlarmControlPanelEntityFeature.ARM_AWAY | AlarmControlPanelEntityFeature.TRIGGER
        )
        if self.coordinator.config.map_extra_alarm_modes is True:
            self._attr_supported_features = (
                self._attr_supported_features
                | AlarmControlPanelEntityFeature.ARM_CUSTOM_BYPASS
                | AlarmControlPanelEntityFeature.ARM_NIGHT
                | AlarmControlPanelEntityFeature.ARM_VACATION
            )

    @property
    def guard_mode_metadata(self) -> Metadata:
        """Get guard mode metadata for device"""
        return self.product.metadata[MessageField.GUARD_MODE.value]

    async def _set_guard_mode(self, target_mode: CurrentModeToState):
        await self.product.set_property(self.guard_mode_metadata, target_mode.value)

    async def async_alarm_disarm(self, code: str | None = None) -> None:
        await self._set_guard_mode(CurrentModeToState.DISARMED)

    async def async_alarm_arm_home(self, code: str | None = None) -> None:
        await self._set_guard_mode(CurrentModeToState.HOME)

    async def async_alarm_arm_away(self, code: str | None = None) -> None:
        await self._set_guard_mode(CurrentModeToState.AWAY)

    async def async_alarm_arm_custom_bypass(self, code: str | None = None) -> None:
        await self._set_guard_mode(CurrentModeToState.CUSTOM_BYPASS)

    async def async_alarm_arm_night(self, code: str | None = None) -> None:
        await self._set_guard_mode(CurrentModeToState.NIGHT)

    async def async_alarm_arm_vacation(self, code: str | None = None) -> None:
        await self._set_guard_mode(CurrentModeToState.VACATION)

    async def async_alarm_trigger(self, code: str | None = None) -> None:
        """trigger alarm for a duration on alarm control panel but there is no change in current mode"""
        await self.product.trigger_alarm()

    async def async_alarm_trigger_with_duration(self, duration: int = 10) -> None:
        """trigger alarm for a duration on alarm control panel but there is no change in current mode"""
        await self.product.trigger_alarm(duration)

    async def async_reset_alarm(self) -> None:
        """reset ongoing alarm but there is no change in current mode"""
        await self.product.reset_alarm()

    @property
    def state(self):
        alarm_delayed = get_child_value(self.product.properties, "alarmDelay", 0)
        if alarm_delayed > 0:
            return CurrentModeToStateValue.ALARM_DELAYED.value
        triggered = get_child_value(self.product.properties, "alarm")
        if triggered is True:
            return CurrentModeToStateValue.TRIGGERED.value
        current_mode = get_child_value(self.product.properties, self.metadata.name, -1)
        if current_mode in CUSTOM_CODES:
            position = CUSTOM_CODES.index(current_mode)
            if position == 0:
                return self.coordinator.config.name_for_custom1
            if position == 1:
                return self.coordinator.config.name_for_custom2
            if position == 2:
                return self.coordinator.config.name_for_custom3
        return CurrentModeToStateValue[CurrentModeToState(current_mode).name].value
